!--------------------------------------------------------------------------------
! Data-push binning in 3-D.
!--------------------------------------------------------------------------------
module bin3  
  use adj_mod
	use morgan_util
	use dottest
	use chain0_mod

  implicit none

  integer, private, save :: m1, m2, m3
  real,    private, save :: o1, d1, o2, d2, o3, d3, wtpow

  real, private, dimension(:),   pointer, save :: tmpbin, wtbin
  real, private, dimension(:,:), pointer, save :: xyz, xyzf 
	logical, private, save :: verbose, normlz
	logical, parameter, private, save :: T=.true., F=.false.

  contains

!--------------------------------------------------------------------------------
subroutine bin3_init( m, o, d, xyz_in, norm, verb, foldpow )

	integer :: m(3), id
	real    :: o(3), d(3), foldpow
	real, dimension (:,:) :: xyz_in 
	logical  :: norm, verb
	optional :: norm, verb, foldpow

	verbose = F;  if(present(verb)) verbose=verb
	normlz  = F;  if(present(norm)) normlz=norm
	wtpow = 1.0;  if(present(foldpow)) wtpow=foldpow
	m1 = m(1);  m2 = m(2);  m3 = m(3)
	o1 = o(1);  o2 = o(2);  o3 = o(3)
	d1 = d(1);  d2 = d(2);  d3 = d(3)
	allocate( xyz(size(xyz_in,dim=1),3) )
	xyz = xyz_in
	if( normlz) then
		allocate( wtbin(m1*m2*m3), tmpbin(m1*m2*m3) )
		call bin3_weight_init( wtbin )
	end if
end subroutine bin3_init
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
subroutine bin3_weight_init( wt )
	real, dimension(:)  :: wt
	integer :: id, im123, im(3)

	wt = 0.0
	do id=1,size(xyz,dim=1)
		im(1) = nint( (xyz(id,1)-o1)/d1 + 1)
		im(2) = nint( (xyz(id,2)-o2)/d2 + 1)
		im(3) = nint( (xyz(id,3)-o3)/d3 + 1)
		if( all(im >= (/1,1,1/)) .AND. all(im <= (/m1,m2,m3/)) ) then
			im123 = grid2line3( (/m1,m2,m3/), im )
			wt(im123) = wt(im123) + 1.0
		end if
	end do

	wt = wt**wtpow

end subroutine bin3_weight_init
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
subroutine get_bin3_interp_wt( wt )
  real, dimension(:)  :: wt
	if( .NOT. normlz) call seperr("get_bin3_interp_wt(): must have norm=T")
  wt = wtbin
end subroutine get_bin3_interp_wt
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
integer function bin3_core( adj, add, model, data )
	logical, intent(IN) :: adj,add 
	real, dimension(:)  :: data, model 
	integer :: id, im123, im(3)

	call adjnull( adj, add, model, data )

	if(verbose) call oper_stats( "bin3_core", T, model, data, adj)

	do id=1,size(data)
		im(1) = nint( (xyz(id,1)-o1)/d1 + 1)
		im(2) = nint( (xyz(id,2)-o2)/d2 + 1)
		im(3) = nint( (xyz(id,3)-o3)/d3 + 1)

		if( all(im >= (/1,1,1/)) .AND. all(im <= (/m1,m2,m3/)) ) then
			im123 = grid2line3( (/m1,m2,m3/), im )
			if( adj) then
				model(im123) = model(im123) + data(id)
			else
				data(id) = data(id) + model(im123)
			end if
		end if
	end do

	if(verbose) call oper_stats( "bin3_core", F, model, data, adj)

end function bin3_core
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
integer function bin3_wt( adj, add, model, data )
	logical, intent(IN) :: adj,add 
	real, dimension(:)  :: model, data 

	call adjnull( adj, add, model, data )

	if(verbose) call oper_stats( "bin3_wt", T, model, data, adj)

	if( adj) then
		where( wtbin > 1e-3 )  model = model + data/wtbin
	else
		where( wtbin > 1e-3 )  data = data + model/wtbin
	end if

	if(verbose) call oper_stats( "bin3_wt", F, model, data, adj)
end function bin3_wt
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
integer function bin3_exec( adj, add, model, data )
	logical, intent(IN) :: adj,add 
	real, dimension(:)  :: model, data 
	integer             :: stat

	call adjnull( adj, add, model, data )

	if(verbose) call oper_stats( "bin3_exec", T, model, data, adj)

	if(normlz) then
		call chain0( bin3_core, bin3_wt, adj, add, model, data, tmpbin)
	else
		stat = bin3_core( adj, add, model, data)
	end if

	if(verbose) call oper_stats( "bin3_exec", F, model, data, adj)
end function bin3_exec
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
subroutine bin3_close()
	if(normlz) then
		deallocate( xyz, wtbin, tmpbin)
	else
		deallocate( xyz )
	end if
end subroutine bin3_close
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
subroutine get_bin3_fold(fold)
	real, dimension(:) :: fold, d_ones
	integer            :: stat

	allocate( d_ones(size(xyz,dim=1)) )
	d_ones = 1.0
	stat = bin3_core( T, F, fold, d_ones)
	deallocate( d_ones) 
end subroutine get_bin3_fold
!--------------------------------------------------------------------------------

!--------------------------------------------------------------------------------
subroutine bin3_test()
	real :: num
	integer :: n1, n2, n3, nd, id, nlev
	real, dimension(:,:), pointer :: xyz_test

	n1=40;  n2=20;  n3=1;  nd = n1*n2*n3/10

	allocate( xyz_test(nd,3) )

	do id=1,nd
		call random_number(num)
		xyz_test(id,1) = (nint(num*n1)-1)*1.0
		call random_number(num)
		xyz_test(id,2) = (nint(num*n2)-1)*1.0
		xyz_test(id,3) = 0.0
	end do
	call bin3_init( (/n1,n2,n3/), (/0.0,0.0,0.0/), (/1.0,1.0,1.0/), xyz_test,     &
 	                norm=T                                                        )
	call bin3_dottest()
	call bin3_close()
	deallocate( xyz_test)
end subroutine bin3_test
!--------------------------------------------------------------------------------


!--------------------------------------------------------------------------------
subroutine bin3_dottest()
	real :: dot1(2), dot2(2)
	write(0,*) '========= Dot Product Testing bin3_exec() ================'
	call dot_test( bin3_exec, m1*m2*m3, size(xyz,dim=1), dot1, dot2 )
	write(0,*) '========= Dot Product Testing bin3_wt() =================='
	call dot_test( bin3_wt, m1*m2*m3, m1*m2*m3, dot1, dot2 )
	write(0,*) '========= Dot Product Testing bin3_core() ================'
	call dot_test( bin3_core, m1*m2*m3, size(xyz,dim=1), dot1, dot2 )
	write(0,*) '==========================================================='

end subroutine bin3_dottest
!--------------------------------------------------------------------------------

end module bin3
