! ---------------------------------------------------------------------------
!  puck_irreg.f90 - Created by Morgan Brown
!           Thu Aug 21 21:37:15 2003.
! ---------------------------------------------------------------------------
!       Usage: puck_irreg.mod 
!  Parameters: 
! Description: 
! ---------------------------------------------------------------------------
module puck_irreg

use adj_mod

implicit none

integer, private, save     :: nd, nt, ntw, nnbr
real, private, save        :: maxdist1, maxdist2, dt, dx, dy
real, dimension(:,:), allocatable, private, save :: xy, dd
logical, parameter, private, save :: T = .true., F = .false.

contains

!-------------------------------------------------------------------------------
subroutine puck_irreg_init( xy_in, nt_in, ntw_in, d_in, maxdist1_in,           &
                            maxdist2_in, dt_in, nnbr_in                        )
	real, dimension(:,:) :: xy_in, d_in
	integer              :: nt_in, ntw_in, nnbr_in
	real                 :: maxdist1_in, maxdist2_in, dt_in

	nt = nt_in                                          ! # Time samples
	ntw = ntw_in                                        ! Puck window in time
	nnbr = nnbr_in                                      ! # neighbor traces to use
	nd = size(xy_in,dim=1)                              ! # irregular traces
	allocate( xy(nd,2), dd(nt,nd) )
	xy = xy_in                                          ! Trace locations
	dd = d_in                                           ! Input traces
	dt = dt_in                                          ! Time sampling
	maxdist1 = maxdist1_in                              ! Valid trace threshold
	maxdist2 = maxdist2_in                              ! Valid trace threshold
end subroutine puck_irreg_init
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
subroutine puck_irreg_close( )
	deallocate( xy, dd)
end subroutine puck_irreg_close
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Loop over all time:
!  Loop over all traces:
!   Loop over all traces except the one above:
!     Solve overdetermined puck problem for px,py at this time/output location
!-------------------------------------------------------------------------------
subroutine puck_irreg( dip)
	real, dimension(:,:,:) :: dip
	real    :: a, b, c, d, g, h, xi, yi, pij, dij, det, ux, ut, wij, xij, yij,   &
	           p, coh, dn(nnbr), tmpr(nnbr)
	integer :: it, id, jd, ct, itw, jt0, jt1, ntmp, inbr, tmpi(nnbr), kd
	integer :: neighbors(nd,nnbr)
	logical :: done

dx = 0.01
dy = 0.02
!dx = 1.00
!dy = 1.00

!	p = -1.0
	p = 0.
	dip = 0.0

	do id=1,nd                      ! Find two nearest traces to each input trace
		dn = 1e30
		xi = xy(id,1)
		yi = xy(id,2)
		do inbr=1,nnbr
			neighbors(id,inbr) = inbr
		end do
		do jd=1,nd
			done = F
			if( id == jd ) cycle
			xij = xy(jd,1) - xi
			yij = xy(jd,2) - yi
			dij = sqrt( xij*xij + yij*yij )

			do inbr=nnbr-1,2,-1
				if( dij < dn(inbr) .AND. dij > dn(inbr-1) ) then
					tmpi(inbr+1:nnbr) = neighbors(id,inbr:nnbr-1)
					neighbors(id,inbr) = jd
					neighbors(id,inbr+1:nnbr) = tmpi(inbr+1:nnbr)
					tmpr(inbr+1:nnbr) = dn(inbr:nnbr-1)
					dn(inbr) = dij
					dn(inbr+1:nnbr) = tmpr(inbr+1:nnbr)
					done = T
				end if
			end do

			if( done) cycle

			if( dij < dn(1) ) then
				tmpi(2:nnbr) = neighbors(id,1:nnbr-1)
				neighbors(id,1) = jd
				neighbors(id,2:nnbr) = tmpi(2:nnbr)
				tmpr(2:nnbr) = dn(1:nnbr-1)
				dn(1) = dij
				dn(2:nnbr) = tmpr(2:nnbr)
				done = T
			end if	

			if( done) cycle

			if( dij < dn(nnbr) .AND. dij > dn(inbr) ) then
				neighbors(id,nnbr) = jd
				dn(nnbr) = dij
			end if
					
		end do
	end do

	do it=ntw+1,nt-1-ntw                                 ! Loop over all times
		do id=1,nd                                         ! Loop over output traces
			xi = xy(id,1)
			yi = xy(id,2)
			a=0; b=0; c=0; d=0; g=0; h=0
			do jd = 1,nnbr
				kd = neighbors(id,jd)
				if( id==kd ) cycle
				xij = xy(kd,1) - xi
				yij = xy(kd,2) - yi
				dij = sqrt( xij*xij + yij*yij )
				if( dij/sqrt(dx*dx+dy*dy) < 1e-6 ) cycle
				ut = 0; ux=0
				do itw=-ntw,ntw
					jt0 = it + itw
					jt1 = it + itw + 1
					ut = ut - dd(jt0,id) - dd(jt0,kd) + dd(jt1,id) + dd(jt1,kd)
					ux = ux - dd(jt0,id) + dd(jt0,kd) - dd(jt1,id) + dd(jt1,kd)
				end do
				if( abs(ut) < 1e-6 .AND. abs(ux)>1 ) cycle
				coh = sqrt((ux/ut)*(ux/ut))
				wij = coh*(dij**p)
				pij = -ux/ut
				a = a + xij*xij*wij*wij/(dt*dt)        ! Hessian = [ a b ]
				b = b + xij*yij*wij*wij/(dt*dt)        !           [ c d ]
				d = d + yij*yij*wij*wij/(dt*dt)        ! Note dij**p wght
				g = g + xij*pij*wij/dt                           ! A'*W'*d(1)
				h = h + yij*pij*wij/dt                           ! A'*W'*d(2)
			end do
			c = b                                                  ! A'A is symmetric

			det = a*d-b*c
			if( det>1e-6 ) then 
				dip(it,id,1) = (  d*g - b*h )/det
				dip(it,id,2) = ( -c*g + a*h )/det
			else
				dip(it,id,1) = 0.0
				dip(it,id,2) = 0.0
			end if

		end do

	end do

end subroutine puck_irreg
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
integer function puck_irreg_exec( adj, add, model, data)
	logical                     :: adj, add 
	real, dimension(:), pointer :: model, data 

	call adjnull( adj, add, model, data)

	if( adj) then

	else

	end if

	puck_irreg_exec = 0

end function puck_irreg_exec
!-------------------------------------------------------------------------------


end module puck_irreg
