/*$

=head1 NAME

Dip3

=head1 DESCRIPTION

Estimating smooth local dips in 2-D and 3-D
  
=head1 SYNOPSIS

< data.H Dip3 > dip.H [eps=1 lam=1 niter=5 p0=0 order=1 nj1=1 nj2=1 sign=0 verb=0] 
 
=head1 INPUT  PARAMETERS

=over 4

=item verb                       - integer

[0] - verbosity flag

=item eps                        - real

[1] - vertical smoothness

=item lam                        - real

[1] - lateral smoothness

=item niter                      - integer

[5] - number of iterations for dip estimation

=item p0                         - real

[0] - initial dip

=item order                      - integer

[1] - filter size (1, 2, or 3)

=item nj1                        - integer

[1] - filter stretching in the (1,2) plane

=item nj2                        - integer

[1] - filter stretching in the (1,3) plane

=item sign                       - integer

[0] - keep the dip sign constant (needs non-zero p0)

=back
 
=head1 SEE ALSO


=head1 COMMENTS

Outputs two cubes (n4=2) for 3-D input (t-x and t-y dips).

The output dip is dimensionless. Multiply by d1/d2 to convert it to 
the appropriate units.

Processes the data and dip cubes in-core.

Use smaller niter and order for efficiency, larger for accuracy.

Use nj1 and/or nj2 for handling aliased dips.

=cut

>*/

/*
****************************************************************************
*  Copyright (c) 2003 
*  All Rights Reserved                                                     *
****************************************************************************
Author: Sergey Fomel <sergey@sep.stanford.edu>
*/ 

#include <sepcube.h>

#include "dip3.h"
#include "seperr.h"

#define ESIZE 4

int main (int argc, char *argv[])
{
    int n1,n2,n3, niter, nw, nj1, nj2, two=2, esize, verb, i3, i2, i1, sign;
    float eps, lam, p0;
    float ***u, ***p;

    initpar(argc,argv);

#ifndef NODOC
    if( !( noheader() && redout())) doc(SOURCE);
#endif

    if (hetch("n1","d",&n1) == 0) seperr("n1 must be supplied\n");
    if (hetch("n2","d",&n2) == 0) seperr("n1 must be supplied\n");
    if (hetch("n3","d",&n3) == 0) seperr("n1 must be supplied\n");

    if (n3 > 1) putch("n4","d",&two);

    if (getch("niter","d",&niter) == 0) niter=5;
    if (getch("eps","f",&eps) == 0) eps=1.; eps = eps*eps; 
    if (getch("lam","f",&lam) == 0) lam=1.; lam = lam*lam;

    if (getch("p0","f",&p0) == 0) p0=0.;
    if (getch("order","d",&nw) == 0) nw=1;
    if (nw < 1 || nw > 3) seperr ("nw must be between 1 and 3\n");
    if (getch("nj1","d",&nj1) == 0) nj1=1;
    if (getch("nj2","d",&nj2) == 0) nj2=1;

    if((0==hetch("esize","i",&esize)) || (esize != ESIZE))
	seperror("esize must be %d\n",ESIZE);

    if(0==getch("verb","i",&verb)) verb = 0;
    if(0==getch("sign","i",&sign)) sign = 0;
    hclose ();

    /* initialize dip estimation */
    dip3_init(n1, n2, n3, eps, lam, sign);

    u = (float***) sepalloc3(n1,n2,n3,sizeof(float));
    p = (float***) sepalloc3(n1,n2,n3,sizeof(float));

    for (i3=0; i3 < n3; i3++) {
	for (i2=0; i2 < n2; i2++) {
	    /* read data */
	    sreed("in",u[i3][i2],n1*esize);
	    /* initialize dip */
	    for(i1=0; i1 < n1; i1++) {
		p[i3][i2][i1] = p0;
	    }
	}
    }
  
    /* estimate t-x dip */
    dip3(1, niter, nw, nj1, verb, u, p);

    for (i3=0; i3 < n3; i3++) {
	for (i2=0; i2 < n2; i2++) {
	    srite("out",p[i3][i2],n1*esize);
	}
    }

    if (n3 > 1) {
	/* estimate t-y dip */
	dip3(2, niter, nw, nj2, verb, u, p);
	for (i3=0; i3 < n3; i3++) {
	    for (i2=0; i2 < n2; i2++) {
		srite("out",p[i3][i2],n1*esize);
	    }
	}
    }

    /* free memory */
    dip3_close();
    free3 (n3,n2, (void***) u);
    free3 (n3,n2, (void***) p);
  
    exit (0);
}
